/* Bluetooth stack headers */
#include "bg_types.h"
#include "native_gecko.h"
#include "gatt_db.h"
#include "app.h"
#include "gpiointerrupt.h"

#define TIMER_ID_READ 2

/* Print boot message */
static void bootMessage(struct gecko_msg_system_boot_evt_t *bootevt);
static void Button_Event(const uint8_t pin);
static uint32_t readPasskey(void);

/* Flag for indicating DFU Reset must be performed */
static uint8_t boot_to_dfu = 0;

uint8_t *addr, *advData, switch_value;
uint8_t i, advData_len, packet_type, conn_handle;
uint32_t passkey; char c;
bd_addr serverAddress;

/* Main application */
void appMain(gecko_configuration_t *pconfig)
{
#if DISABLE_SLEEP > 0
  pconfig->sleep.flags = 0;
#endif

  /* Initialize debug prints. Note: debug prints are off by default. See DEBUG_LEVEL in app.h */
  initLog();

  /* Initialize stack */
  gecko_init(pconfig);

 // memcpy(serverAddress.addr, (uint8_t[]){0x37, 0x86, 0x7B, 0x9F, 0xFD, 0x90}, 6);
  memcpy(serverAddress.addr, (uint8_t[]){0x1e, 0x84, 0x8c, 0x57, 0x0b, 0x00}, 6);	// BGM board
  GPIO_PinModeSet(BSP_LED0_PORT, BSP_LED0_PIN, gpioModePushPull, 0);				// red LED
  GPIO_PinModeSet(BSP_LED1_PORT, BSP_LED1_PIN, gpioModePushPull, 0);				// green LED
  GPIO_PinModeSet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN, gpioModeInputPull, 1);
  GPIO_PinModeSet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN, gpioModeInputPull, 1);
  GPIO_ExtIntConfig(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN, BSP_BUTTON1_PIN, false, true, true);
  GPIOINT_Init();												// init GPIOINT driver
  GPIOINT_CallbackRegister(BSP_BUTTON1_PIN, Button_Event);   	// register callback function

  while (1) {
    /* Event pointer for handling events */
    struct gecko_cmd_packet* evt;

    /* if there are no events pending then the next call to gecko_wait_event() may cause
     * device go to deep sleep. Make sure that debug prints are flushed before going to sleep */
    if (!gecko_event_pending()) {
    	flushLog();
    }

    /* Check for stack event. This is a blocking event listener. If you want non-blocking please see UG136. */
    evt = gecko_wait_event();

    /* Handle events */
    switch (BGLIB_MSG_ID(evt->header)) {
      case gecko_evt_system_boot_id:
    	  bootMessage(&(evt->data.evt_system_boot));
//    	  gecko_cmd_sm_configure(0x07,sm_io_capability_keyboarddisplay);	// allow all connections
    	  gecko_cmd_sm_configure(0x07, sm_io_capability_displayyesno);
    	  gecko_cmd_sm_set_bondable_mode(1);								// enable new bondings
    	  if (!GPIO_PinInGet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN)) {			// erase all bondings
    		  gecko_cmd_sm_delete_bondings();								// if BUTTON1 is pressed
    		  printf("All bondings are erased. Release BUTTON1...\r\n");
    	  }

    	  gecko_cmd_le_gap_set_discovery_timing(le_gap_phy_1m, 160, 160);				// set scanning period 10ms
    	  gecko_cmd_le_gap_set_discovery_type(le_gap_phy_1m, 1);						// do active scanning
    	  gecko_cmd_le_gap_set_conn_timing_parameters(80, 100, 0, 1000, 0, 0xffff);		// set connection interval 100ms
    	  gecko_cmd_le_gap_start_discovery(le_gap_phy_1m, le_gap_discover_observation);	// start discovering all devices
    	  printLog("Scanning started\r\n");
      break;

      case gecko_evt_le_connection_opened_id:
    	  GPIO_PinOutSet(BSP_LED0_PORT, BSP_LED0_PIN);									// turn on LED
    	  printLog("Connected\r\n");
    	  conn_handle = evt->data.evt_le_connection_opened.connection;
    	  gecko_cmd_sm_increase_security(conn_handle);
      break;

      case gecko_evt_le_connection_closed_id:
    	  GPIO_PinOutClear(BSP_LED0_PORT, BSP_LED0_PIN);								// turn off LED
    	  printLog("Connection closed, reason: 0x%2.2x\r\n", evt->data.evt_le_connection_closed.reason);
    	  /* Check if need to boot to OTA DFU mode */
    	  if (boot_to_dfu) {
          /* Enter to OTA DFU mode */
    		  gecko_cmd_system_reset(2);
    	  } else {
          /* Restart advertising after client has disconnected */
    		  gecko_cmd_le_gap_start_advertising(0, le_gap_general_discoverable, le_gap_connectable_scannable);
    	  }
      break;

      /* Events related to OTA upgrading
         ----------------------------------------------------------------------------- */
      /* Check if the user-type OTA Control Characteristic was written.
       * If ota_control was written, boot the device into Device Firmware Upgrade (DFU) mode. */
      case gecko_evt_gatt_server_user_write_request_id:

        if (evt->data.evt_gatt_server_user_write_request.characteristic == gattdb_ota_control) {
          /* Set flag to enter to OTA mode */
          boot_to_dfu = 1;
          /* Send response to Write Request */
          gecko_cmd_gatt_server_send_user_write_response(
            evt->data.evt_gatt_server_user_write_request.connection,
            gattdb_ota_control,
            bg_err_success);

          /* Close connection to enter to DFU OTA mode */
          gecko_cmd_le_connection_close(evt->data.evt_gatt_server_user_write_request.connection);
        }
      break;

      /* Add additional event handlers as your application requires */
//=========================================================================================================
      case gecko_evt_le_gap_scan_response_id:								// some device is discovered
    	  addr = evt->data.evt_le_gap_scan_response.address.addr;			// get discovered device address
    	  if (!memcmp(addr, serverAddress.addr, 6))	{						// server is found
        	  advData_len = evt->data.evt_le_gap_scan_response.data.len;
        	  advData = evt->data.evt_le_gap_scan_response.data.data;
        	  packet_type = evt->data.evt_le_gap_scan_response.packet_type & 7;
    		  printLog("%02x:%02x:%02x:%02x:%02x:%02x   (%d)   ",
    				  addr[5], addr[4], addr[3], addr[2], addr[1], addr[0], packet_type);
    		  if (packet_type == 4)	{										// wait for scan response
    			  gecko_cmd_le_gap_end_procedure();							// stop discovering
    			  for (i=0; i<advData_len; i++)
    				  printLog("%c", advData[i+2]);
    			  printLog("\r\n");
    			  printLog("Scanning stopped\r\n");
    			  gecko_cmd_le_gap_connect(serverAddress, 0, le_gap_phy_1m); // connect to server
    		  }
    		  else
    			  printLog("\r\n");
    	  }
      break;

      case gecko_evt_sm_passkey_request_id:
    	  printf("Enter passkey: ");
    	  passkey = readPasskey();
    	  gecko_cmd_sm_enter_passkey(conn_handle, passkey);
      break;

      case gecko_evt_sm_confirm_passkey_id:
    	  printf("Do you see this passkey on the other device: %06u? (y/n)\r\n",
    			  (int)evt->data.evt_sm_confirm_passkey.passkey);
    	  c = RETARGET_ReadChar();
    	  while (c != 'y' && c != 'n' && c != 'Y' && c != 'N'){
    		  c = getchar();
    	  }
    	  if (c == 'y' || c == 'Y'){
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 1);
    		  printf("Waiting for other device to confirm...\r\n");
    	  }
    	  else	{
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 0);
    	  }
      break;

      case gecko_evt_sm_bonded_id:
    	  printf("bond success\r\n");
      break;

      case gecko_evt_sm_bonding_failed_id:
    	  printf("bonding failed, reason 0x%2X\r\n", evt->data.evt_sm_bonding_failed.reason);
    	  gecko_cmd_le_connection_close(evt->data.evt_sm_bonding_failed.connection);
      break;

      case gecko_evt_system_external_signal_id:					// button event
    	  if ((evt->data.evt_system_external_signal.extsignals & 0xFFFF) == (1<<BSP_BUTTON1_PIN)) {
    		  GPIO_PinOutToggle(BSP_LED1_PORT, BSP_LED1_PIN);
    		  switch_value = GPIO_PinInGet(BSP_LED1_PORT, BSP_LED1_PIN);
    		  gecko_cmd_gatt_write_characteristic_value(conn_handle, 26, 1, &switch_value);
    	  }
      break;

      default:
      break;
    }
  }
}

/* Print stack version and local Bluetooth address as boot message */
static void bootMessage(struct gecko_msg_system_boot_evt_t *bootevt)
{
#if DEBUG_LEVEL
  bd_addr local_addr;
  int i;

  printLog("\r\nstack version: %u.%u.%u\r\n", bootevt->major, bootevt->minor, bootevt->patch);
  local_addr = gecko_cmd_system_get_bt_address()->address;

  printLog("local BT device address: ");
  for (i = 0; i < 5; i++) {
    printLog("%2.2x:", local_addr.addr[5 - i]);
  }
  printLog("%2.2x\r\n", local_addr.addr[0]);
#endif
}

static void Button_Event(const uint8_t pin)
{
	if (!GPIO_PinInGet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN)) {
	      gecko_external_signal((1<<BSP_BUTTON1_PIN) | (1<<16));
	}
}

static uint32_t readPasskey(void)
{
	uint8_t i = 0;
	uint32_t d = 0;
	char c;

	while (i < 8)
	{
		c = RETARGET_ReadChar();
		if (c == 13)
		{
			printf("\r\n");
			break;
		}
		if ('0' <= c && c <= '9')
		{
			printf("%c", c);
			d = d*10 + (c - '0');
			i++;
		}
	}
	return d;
}
