/* Bluetooth stack headers */
#include "bg_types.h"
#include "native_gecko.h"
#include "gatt_db.h"

#include "app.h"

/* Print boot message */
static void bootMessage(struct gecko_msg_system_boot_evt_t *bootevt);
uint32_t makePasskeyFromAddress(bd_addr address);
uint32_t makeRandomPasskey(void);

/* Flag for indicating DFU Reset must be performed */
static uint8_t boot_to_dfu = 0;
uint32 passkey = 0;
bd_addr public_address;
uint8_t bonding;
uint8_t *addr, conn_handle; char c;
uint8_t switchState;

/* Main application */
void appMain(gecko_configuration_t *pconfig)
{
#if DISABLE_SLEEP > 0
  pconfig->sleep.flags = 0;
#endif

  // Init LED and buttons  LED0=right  LED1=left
//    GPIO_PinModeSet(BSP_LED0_PORT, BSP_LED0_PIN, gpioModePushPull, 1);
//    GPIO_PinModeSet(BSP_LED1_PORT, BSP_LED1_PIN, gpioModePushPull, 1);
    GPIO_PinModeSet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN, gpioModeInputPull, 1);
    GPIO_PinModeSet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN, gpioModeInputPull, 1);

  /* Initialize debug prints. Note: debug prints are off by default. See DEBUG_LEVEL in app.h */
  initLog();

  /* Initialize stack */
  gecko_init(pconfig);

  while (1) {
    /* Event pointer for handling events */
    struct gecko_cmd_packet* evt;

    /* if there are no events pending then the next call to gecko_wait_event() may cause
     * device go to deep sleep. Make sure that debug prints are flushed before going to sleep */
    if (!gecko_event_pending()) {
      flushLog();
    }

    /* Check for stack event. This is a blocking event listener. If you want non-blocking please see UG136. */
    evt = gecko_wait_event();

    /* Handle events */
    switch (BGLIB_MSG_ID(evt->header)) {
      /* This boot event is generated when the system boots up after reset.
       * Do not call any stack commands before receiving the boot event.
       * Here the system is set to start advertising immediately after boot procedure. */
      case gecko_evt_system_boot_id:
    	  bootMessage(&(evt->data.evt_system_boot));
    	  public_address = gecko_cmd_system_get_bt_address()->address;
    	  passkey = makeRandomPasskey();
    	  if (GPIO_PinInGet(BSP_BUTTON0_PORT, BSP_BUTTON0_PIN)){
    		  printf("Connecting only to bonded devices\r\n");
    		  gecko_cmd_sm_configure(0x17,sm_io_capability_displayonly);	// connect only with bonded devices
//    		  gecko_cmd_sm_configure(0x17, sm_io_capability_displayyesno);
    		  gecko_cmd_sm_set_bondable_mode(0);							// disable new bondings
    	  }
    	  else {
    		  printf("All connections are enabled. Release BUTTON0...\r\n");
    		  gecko_cmd_sm_configure(0x07,sm_io_capability_displayonly);	// allow all connections
//    		  gecko_cmd_sm_configure(0x07, sm_io_capability_displayyesno);
    		  gecko_cmd_sm_set_bondable_mode(1);							// enable new bondings
    	  }
    	  gecko_cmd_sm_set_passkey(passkey);								// set passkey
    	  if (!GPIO_PinInGet(BSP_BUTTON1_PORT, BSP_BUTTON1_PIN)) {			// erase all bondings
    		  gecko_cmd_sm_delete_bondings();								// if BUTTON1 is pressed
    		  printf("All bondings are erased. Release BUTTON1...\r\n");
    	  }
    	  gecko_cmd_le_gap_set_advertise_timing(0, 160, 160, 0, 0);
    	  gecko_cmd_sm_list_all_bondings();
    	  printf("Bonded devices:\r\n");
      break;

      case gecko_evt_le_connection_opened_id:
    	  printLog("connection opened\r\n");
//    	  gecko_cmd_sm_increase_security(evt->data.evt_le_connection_opened.connection);
    	  conn_handle = evt->data.evt_le_connection_opened.connection;
    	  GPIO_PinModeSet(BSP_LED0_PORT, BSP_LED0_PIN, gpioModePushPull, 1);
    	  GPIO_PinModeSet(BSP_LED1_PORT, BSP_LED1_PIN, gpioModePushPull, 1);
          GPIO_PinOutClear(BSP_LED0_PORT, BSP_LED0_PIN);					// turn on LED
      break;

      case gecko_evt_le_connection_closed_id:
          GPIO_PinOutSet(BSP_LED0_PORT, BSP_LED0_PIN);						// turn off LED
    	  printLog("connection closed, reason: 0x%2.2x\r\n", evt->data.evt_le_connection_closed.reason);
    	  /* Check if need to boot to OTA DFU mode */
    	  if (boot_to_dfu) {
    		  /* Enter to OTA DFU mode */
    		  gecko_cmd_system_reset(2);
    	  } else {
    		  /* Restart advertising after client has disconnected */
    		  gecko_cmd_le_gap_start_advertising(0, le_gap_general_discoverable, le_gap_connectable_scannable);
    	  }
      break;

      /* Events related to OTA upgrading
         ----------------------------------------------------------------------------- */

      /* Check if the user-type OTA Control Characteristic was written.
       * If ota_control was written, boot the device into Device Firmware Upgrade (DFU) mode. */
      case gecko_evt_gatt_server_user_write_request_id:
        if (evt->data.evt_gatt_server_user_write_request.characteristic == gattdb_ota_control) {
          /* Set flag to enter to OTA mode */
          boot_to_dfu = 1;
          /* Send response to Write Request */
          gecko_cmd_gatt_server_send_user_write_response(
            evt->data.evt_gatt_server_user_write_request.connection,
            gattdb_ota_control,
            bg_err_success);
          /* Close connection to enter to DFU OTA mode */
          gecko_cmd_le_connection_close(evt->data.evt_gatt_server_user_write_request.connection);
        }
        break;

      /* Add additional event handlers as your application requires */
//==============================================================================================
      case gecko_evt_sm_passkey_display_id:
    	  printf("passkey: %d\r\n", (int)evt->data.evt_sm_passkey_display.passkey);
      break;

      case gecko_evt_sm_bonded_id:
    	  printf("bond success\r\n");
      break;

      case gecko_evt_sm_bonding_failed_id:
    	  printf("bonding failed, reason 0x%2X\r\n", evt->data.evt_sm_bonding_failed.reason);
    	  gecko_cmd_le_connection_close(evt->data.evt_sm_bonding_failed.connection);
      break;

      case gecko_evt_gatt_server_attribute_value_id:
    	  if (evt->data.evt_gatt_server_attribute_value.attribute == gattdb_digital){
    		  switchState = evt->data.evt_gatt_server_attribute_value.value.data[0];
    		  if (switchState)
    			  GPIO_PinOutClear(BSP_LED1_PORT, BSP_LED1_PIN);
    		  else
    		      GPIO_PinOutSet(BSP_LED1_PORT, BSP_LED1_PIN);
    	  }
      break;

      case gecko_evt_sm_list_bonding_entry_id:
    	  bonding = evt->data.evt_sm_list_bonding_entry.bonding;
    	  addr = evt->data.evt_sm_list_bonding_entry.address.addr;
    	  printf("%d: %02x:%02x:%02x:%02x:%02x:%02x \r\n", bonding,
    	      		addr[5], addr[4], addr[3], addr[2], addr[1], addr[0]);
      break;

      case gecko_evt_sm_list_all_bondings_complete_id:
    	  printf("End of list\r\n");
       	  printf("boot event - starting advertising\r\n");
          gecko_cmd_le_gap_start_advertising(0, le_gap_general_discoverable, le_gap_connectable_scannable);
      break;

      // events for Numeric Comparison method
      case gecko_evt_sm_confirm_bonding_id:
    	  gecko_cmd_sm_bonding_confirm(conn_handle, 1);
      break;

          // Event raised by the security manager when a passkey needs to be confirmed
      case gecko_evt_sm_confirm_passkey_id:
    	  printf("Do you see this passkey on the other device: %06u? (y/n)\r\n",
    			  (int)evt->data.evt_sm_confirm_passkey.passkey);
    	  c = RETARGET_ReadChar();
    	  while (c != 'y' && c != 'n' && c != 'Y' && c != 'N'){
    		  c = getchar();
    	  }
    	  if (c == 'y' || c == 'Y'){
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 1);
    		  printf("Waiting for other device to confirm...\r\n");
    	  }
    	  else	{
    		  gecko_cmd_sm_passkey_confirm(conn_handle, 0);
    	  }
      break;


      default:
      break;
    }
  }
}

/* Print stack version and local Bluetooth address as boot message */
static void bootMessage(struct gecko_msg_system_boot_evt_t *bootevt)
{
#if DEBUG_LEVEL
  bd_addr local_addr;
  int i;

  printLog("\r\nStack version: %u.%u.%u\r\n", bootevt->major, bootevt->minor, bootevt->patch);
  local_addr = gecko_cmd_system_get_bt_address()->address;

  printLog("Local BT device address: ");
  for (i = 0; i < 5; i++) {
    printLog("%2.2x:", local_addr.addr[5 - i]);
  }
  printLog("%2.2x\r\n", local_addr.addr[0]);
#endif
}

uint32 makePasskeyFromAddress(bd_addr address)
{
	uint32 passkey=0;
	for(int i=0; i < sizeof(bd_addr); i++){
		passkey += address.addr[i]<<8;
	}
	return passkey;
}

uint32_t makeRandomPasskey(void)
{
	CMU_ClockEnable(cmuClock_TRNG0, true);	// enable clock to TRNG
	TRNG0->CONTROL = 1; 					// enable TRNG
	uint32_t n = TRNG0->FIFO; 				// get 6-digit TRN
	n = (n % 899999) + 100000;
	return(n);
}
